# -*- coding: utf-8 -*-
from zvt.contract import IntervalLevel


def to_ccxt_trading_level(trading_level: IntervalLevel):
    return trading_level.value


from .coin_kdata_recorder import *
from .coin_tick_recorder import *
from .coin_recorder import *



# -*- coding: utf-8 -*-
import argparse

from zvt import init_log
from zvt.api import generate_kdata_id, get_kdata_schema
from zvt.contract import IntervalLevel
from zvt.contract.recorder import FixedCycleDataRecorder
from zvt.utils.time_utils import to_pd_timestamp
from zvt.utils.time_utils import to_time_str
from zvt_ccxt.accounts import CCXTAccount
from zvt_ccxt.domain import Coin, CoinKdataCommon
from zvt_ccxt.recorders import to_ccxt_trading_level
from zvt_ccxt.settings import COIN_EXCHANGES, COIN_PAIRS


class CoinKdataRecorder(FixedCycleDataRecorder):
    provider = 'ccxt'

    entity_provider = 'ccxt'
    entity_schema = Coin

    # 只是为了把recorder注册到data_schema
    data_schema = CoinKdataCommon

    def __init__(self,
                 exchanges=['binance'],
                 entity_ids=None,
                 codes=None,
                 batch_size=10,
                 force_update=True,
                 sleeping_time=10,
                 default_size=2000,
                 real_time=False,
                 fix_duplicate_way='ignore',
                 start_timestamp=None,
                 end_timestamp=None,
                 level=IntervalLevel.LEVEL_1DAY,
                 kdata_use_begin_time=True,
                 close_hour=None,
                 close_minute=None,
                 one_day_trading_minutes=24 * 60) -> None:
        self.data_schema = get_kdata_schema(entity_type='coin', level=level)
        self.ccxt_trading_level = to_ccxt_trading_level(level)

        super().__init__('coin', exchanges, entity_ids, codes, batch_size, force_update, sleeping_time,
                         default_size, real_time, fix_duplicate_way, start_timestamp, close_hour, close_minute,
                         end_timestamp, level, kdata_use_begin_time, one_day_trading_minutes)

    def generate_domain_id(self, entity, original_data):
        return generate_kdata_id(entity_id=entity.id, timestamp=original_data['timestamp'], level=self.level)

    def record(self, entity, start, end, size, timestamps):
        start_timestamp = to_time_str(start)

        ccxt_exchange = CCXTAccount.get_ccxt_exchange(entity.exchange)

        if ccxt_exchange.has['fetchOHLCV']:
            limit = CCXTAccount.get_kdata_limit(entity.exchange)

            limit = min(size, limit)

            kdata_list = []

            if CCXTAccount.exchange_conf[entity.exchange]['support_since']:
                kdatas = ccxt_exchange.fetch_ohlcv(entity.code,
                                                   timeframe=self.ccxt_trading_level,
                                                   since=start_timestamp)
            else:
                kdatas = ccxt_exchange.fetch_ohlcv(entity.code,
                                                   timeframe=self.ccxt_trading_level,
                                                   limit=limit)

            for kdata in kdatas:
                current_timestamp = kdata[0]
                if self.level == IntervalLevel.LEVEL_1DAY:
                    current_timestamp = to_time_str(current_timestamp)

                kdata_json = {
                    'timestamp': to_pd_timestamp(current_timestamp),
                    'open': kdata[1],
                    'high': kdata[2],
                    'low': kdata[3],
                    'close': kdata[4],
                    'volume': kdata[5],
                    'name': entity.name,
                    'provider': 'ccxt',
                    'level': self.level.value
                }
                kdata_list.append(kdata_json)

            return kdata_list
        else:
            self.logger.warning("exchange:{} not support fetchOHLCV".format(entity.exchange))


__all__ = ["CoinKdataRecorder"]

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--level', help='trading level', default='1m', choices=[item.value for item in IntervalLevel])
    parser.add_argument('--exchanges', help='exchanges', default='binance', nargs='+',
                        choices=[item for item in COIN_EXCHANGES])
    parser.add_argument('--codes', help='codes', default='EOS/USDT', nargs='+',
                        choices=[item for item in COIN_PAIRS])

    args = parser.parse_args()

    level = IntervalLevel(args.level)

    exchanges = args.exchanges
    if type(exchanges) != list:
        exchanges = [exchanges]

    codes = args.codes
    if type(codes) != list:
        codes = [codes]

    init_log(
        'coin_{}_{}_{}_kdata.log'.format('-'.join(exchanges), '-'.join(codes).replace('/', ''), args.level))

    CoinKdataRecorder(exchanges=exchanges, codes=codes, level=level, real_time=True).run()



# 数字货币
import pandas as pd
from sqlalchemy.ext.declarative import declarative_base

from zvt.contract import EntityMixin
from zvt.contract.register import register_entity, register_schema

CoinMetaBase = declarative_base()


@register_entity(entity_type='coin')
class Coin(CoinMetaBase, EntityMixin):
    __tablename__ = 'coin'

    @classmethod
    def get_trading_dates(cls, start_date=None, end_date=None):
        return pd.date_range(start_date, end_date, freq='D')

    @classmethod
    def could_short(cls):
        return True

    @classmethod
    def get_trading_t(cls):
        return 0

    @classmethod
    def get_trading_intervals(cls):
        return [('00:00,23:59')]


register_schema(providers=['ccxt'], db_name='coin', schema_base=CoinMetaBase)

__all__ = ['Coin']



# -*- coding: utf-8 -*-

# ****** setting for crypto currency ****** #
COIN_EXCHANGES = ["binance", "huobipro"]

# COIN_BASE = ["BTC", "ETH", "XRP", "BCH", "EOS", "LTC", "XLM", "ADA", "IOTA", "TRX", "NEO", "DASH", "XMR",
#                        "BNB", "ETC", "QTUM", "ONT"]

COIN_BASE = ["BTC", "ETH", "EOS"]

COIN_PAIRS = [("{}/{}".format(item, "USDT")) for item in COIN_BASE] + \
             [("{}/{}".format(item, "USD")) for item in COIN_BASE]



# -*- coding: utf-8 -*-
from .context import init_test_context

init_test_context()

from zvt_ccxt.domain import Coin


def test_coin():
    try:
        Coin.record_data()
        Coin.query_data()
    except:
        assert False



# -*- coding: utf-8 -*-
import json

import ccxt

from zvt import zvt_env
from zvt_ccxt.settings import COIN_EXCHANGES


class CCXTAccount(object):
    exchanges = COIN_EXCHANGES
    exchange_conf = {}

    @classmethod
    def init(cls):
        for exchange in cls.exchanges:
            import pkg_resources

            resource_package = 'zvt_ccxt'
            resource_path = 'accounts/{}.json'.format(exchange)
            config_file = pkg_resources.resource_filename(resource_package, resource_path)

            with open(config_file) as f:
                cls.exchange_conf[exchange] = json.load(f)

    @classmethod
    def get_tick_limit(cls, exchange):
        return cls.exchange_conf[exchange]['tick_limit']

    @classmethod
    def get_kdata_limit(cls, exchange):
        return cls.exchange_conf[exchange]['kdata_limit']

    @classmethod
    def get_safe_sleeping_time(cls, exchange):
        return cls.exchange_conf[exchange]['safe_sleeping_time']

    @classmethod
    def get_ccxt_exchange(cls, exchange_str) -> ccxt.Exchange:
        exchange = eval("ccxt.{}()".format(exchange_str))
        exchange.apiKey = cls.exchange_conf[exchange_str]['apiKey']
        exchange.secret = cls.exchange_conf[exchange_str]['secret']
        # set to your proxies if need
        exchange.proxies = {'http': zvt_env['http_proxy'], 'https': zvt_env['https_proxy']}
        return exchange



# -*- coding: utf-8 -*-
# this file is generated by gen_kdata_schema function, dont't change it
from sqlalchemy.ext.declarative import declarative_base

from zvt.contract.register import register_schema
from zvt_ccxt.domain.quotes import CoinKdataCommon

KdataBase = declarative_base()


class Coin5mKdata(KdataBase, CoinKdataCommon):
    __tablename__ = 'coin_5m_kdata'


register_schema(providers=['ccxt'], db_name='coin_5m_kdata', schema_base=KdataBase)

__all__ = ['Coin5mKdata']



# -*- coding: utf-8 -*-
# To use a consistent encoding
from codecs import open
from os import path

# Always prefer setuptools over distutils
from setuptools import setup, find_packages

try:
    # for pip >= 10
    from pip._internal.req import parse_requirements
except ImportError:
    # for pip <= 9.0.3
    from pip.req import parse_requirements

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

# Arguments marked as "Required" below must be included for upload to PyPI.
# Fields marked as "Optional" may be commented out.

install_reqs = parse_requirements("requirements.txt", session=False)

try:
    requirements = [str(ir.req) for ir in install_reqs]
except:
    requirements = [str(ir.requirement) for ir in install_reqs]

setup(
    name="zvt-ccxt",
    version='0.0.6',
    description='ccxt cryptocurrency plugin for zvt ',
    install_requires=requirements,
    entry_points={"zvt": ["ccxt = zvt_ccxt"]},
    py_modules=["zvt_ccxt"],
    packages=find_packages(),
    package_data={
        'zvt_ccxt.accounts': ['*.json']
    },
    long_description=long_description,
    url='https://github.com/zvtvz/zvt-ccxt',
    author='foolcage',
    author_email='5533061@qq.com',
    classifiers=[  # Optional
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Customer Service',
        'Intended Audience :: Education',
        'Intended Audience :: Financial and Insurance Industry',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Office/Business :: Financial :: Investment',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7'
    ],

    keywords='quant zvt cryptocurrency coin btc eos',

    project_urls={  # Optional
        'Bug Reports': 'https://github.com/zvtvz/zvt-ccxt/issues',
        'Funding': 'https://github.com/zvtvz/zvt-ccxt',
        'Say Thanks!': 'https://saythanks.io/to/foolcage',
        'Source': 'https://github.com/zvtvz/zvt-ccxt',
    },

    include_package_data=True,
    long_description_content_type="text/markdown",
)



import pandas as pd

from zvt.contract.api import df_to_db
from zvt.contract.recorder import Recorder
from zvt_ccxt.accounts import CCXTAccount
from zvt_ccxt.domain import Coin
from zvt_ccxt.settings import COIN_EXCHANGES, COIN_PAIRS


class CoinMetaRecorder(Recorder):
    provider = 'ccxt'
    data_schema = Coin

    def __init__(self, batch_size=10, force_update=False, sleeping_time=10, exchanges=COIN_EXCHANGES) -> None:
        super().__init__(batch_size, force_update, sleeping_time)
        self.exchanges = exchanges

    def run(self):
        for exchange_str in self.exchanges:
            exchange = CCXTAccount.get_ccxt_exchange(exchange_str)
            try:
                markets = exchange.fetch_markets()
                df = pd.DataFrame()

                # markets有些为key=symbol的dict,有些为list
                markets_type = type(markets)
                if markets_type != dict and markets_type != list:
                    self.logger.exception("unknown return markets type {}".format(markets_type))
                    return

                aa = []
                for market in markets:
                    if markets_type == dict:
                        name = market
                        code = market

                    if markets_type == list:
                        code = market['symbol']
                        name = market['symbol']

                    if name not in COIN_PAIRS:
                        continue
                    aa.append(market)

                    security_item = {
                        'id': '{}_{}_{}'.format('coin', exchange_str, code),
                        'entity_id': '{}_{}_{}'.format('coin', exchange_str, code),
                        'exchange': exchange_str,
                        'entity_type': 'coin',
                        'code': code,
                        'name': name
                    }

                    df = df.append(security_item, ignore_index=True)

                # 存储该交易所的数字货币列表
                if not df.empty:
                    df_to_db(df=df, data_schema=self.data_schema, provider=self.provider, force_update=True)
                self.logger.info("init_markets for {} success".format(exchange_str))
            except Exception as e:
                self.logger.exception(f"init_markets for {exchange_str} failed", e)


__all__ = ["CoinMetaRecorder"]

if __name__ == '__main__':
    CoinMetaRecorder().run()



from zvt.contract import IntervalLevel
from zvt.domain.quotes.gen_kdata_schema import gen_kdata_schema

if __name__ == '__main__':
    gen_kdata_schema(pkg='zvt_ccxt', providers=['ccxt'], entity_type='coin',
                     levels=[level for level in IntervalLevel])



# -*- coding: utf-8 -*-
# this file is generated by gen_kdata_schema function, dont't change it
from zvt_ccxt.domain.quotes.coin.coin_tick_kdata import *
from zvt_ccxt.domain.quotes.coin.coin_1m_kdata import *
from zvt_ccxt.domain.quotes.coin.coin_5m_kdata import *
from zvt_ccxt.domain.quotes.coin.coin_15m_kdata import *
from zvt_ccxt.domain.quotes.coin.coin_30m_kdata import *
from zvt_ccxt.domain.quotes.coin.coin_1h_kdata import *
from zvt_ccxt.domain.quotes.coin.coin_4h_kdata import *
from zvt_ccxt.domain.quotes.coin.coin_1d_kdata import *
from zvt_ccxt.domain.quotes.coin.coin_1wk_kdata import *
from zvt_ccxt.domain.quotes.coin.coin_1mon_kdata import *


# -*- coding: utf-8 -*-
import argparse

from zvt import init_log
from zvt.api import get_kdata_schema, generate_kdata_id
from zvt.contract import IntervalLevel
from zvt.contract.recorder import FixedCycleDataRecorder
from zvt.utils.time_utils import to_pd_timestamp
from zvt_ccxt.accounts import CCXTAccount
from zvt_ccxt.domain import Coin, CoinTickCommon
from zvt_ccxt.settings import COIN_EXCHANGES, COIN_PAIRS


class CoinTickRecorder(FixedCycleDataRecorder):
    provider = 'ccxt'

    entity_provider = 'ccxt'
    entity_schema = Coin

    # 只是为了把recorder注册到data_schema
    data_schema = CoinTickCommon

    def __init__(self,
                 exchanges=['binance'],
                 entity_ids=None,
                 codes=None,
                 batch_size=10,
                 force_update=True,
                 sleeping_time=10,
                 default_size=2000,
                 real_time=True,
                 fix_duplicate_way='ignore',
                 start_timestamp=None,
                 end_timestamp=None,
                 kdata_use_begin_time=False,
                 close_hour=None,
                 close_minute=None,
                 level=IntervalLevel.LEVEL_TICK,
                 one_day_trading_minutes=24 * 60) -> None:

        self.data_schema = get_kdata_schema(entity_type='coin', level=level)

        super().__init__('coin', exchanges, entity_ids, codes, batch_size, force_update, sleeping_time,
                         default_size, real_time, fix_duplicate_way, start_timestamp, end_timestamp, close_hour,
                         close_minute, IntervalLevel.LEVEL_TICK, kdata_use_begin_time, one_day_trading_minutes)

    def generate_domain_id(self, entity, original_data):
        return generate_kdata_id(entity_id=entity.id, timestamp=original_data['timestamp'], level=self.level)

    def record(self, entity, start, end, size, timestamps):
        if size < 20:
            size = 20

        ccxt_exchange = CCXTAccount.get_ccxt_exchange(entity.exchange)

        if ccxt_exchange.has['fetchTrades']:
            limit = CCXTAccount.get_tick_limit(entity.exchange)

            limit = min(size, limit)

            kdata_list = []

            trades = ccxt_exchange.fetch_trades(entity.code, limit=limit)

            for trade in trades:
                kdata_json = {
                    'name': entity.name,
                    'provider': 'ccxt',
                    # 'id': trade['id'],
                    'level': 'tick',
                    'order': trade['order'],
                    'timestamp': to_pd_timestamp(trade['timestamp']),
                    'price': trade['price'],
                    'volume': trade['amount'],
                    'direction': trade['side'],
                    'order_type': trade['type'],
                    'turnover': trade['price'] * trade['amount']
                }
                kdata_list.append(kdata_json)

            return kdata_list
        else:
            self.logger.warning("exchange:{} not support fetchOHLCV".format(entity.exchange))


__all__ = ["CoinTickRecorder"]

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--exchanges', help='exchanges', default='binance', nargs='+',
                        choices=[item for item in COIN_EXCHANGES])
    parser.add_argument('--codes', help='codes', default='EOS/USDT', nargs='+',
                        choices=[item for item in COIN_PAIRS])

    args = parser.parse_args()

    init_log('coin_tick_kdata.log')

    CoinTickRecorder(codes=['EOS/USDT']).run()



# -*- coding: utf-8 -*-
import zvt
from zvt_ccxt.accounts import CCXTAccount


@zvt.hookimpl
def zvt_setup_env(config: dict):
    return "zvt_ccxt", CCXTAccount.exchange_conf


from zvt_ccxt.domain import *
from zvt_ccxt.recorders import *
